package com.tencent.liteav.demo.player.viewmodel

import android.content.Context
import android.util.Log
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.viewModelScope
import com.tencent.liteav.demo.player.util.PlayerTimerUtil
import com.tencent.liteav.demo.superplayer.SuperPlayerModel
import com.tencent.liteav.demo.superplayer.database.entity.CountDown
import com.tencent.liteav.demo.superplayer.database.entity.History
import com.tencent.liteav.demo.superplayer.database.repo.PlayerRepository
import com.tencent.liteav.demo.superplayer.util.CountDownUtil
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.async
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.text.SimpleDateFormat
import java.util.*

class PlayerViewModel(
    private val courseId: String,
    private val repository: PlayerRepository
) : ViewModel(), PlayerTimerUtil.Listener, CountDownUtil.OnTickListener {

    init {
        PlayerTimerUtil.addListener(this)
    }

    private lateinit var countDown: CountDown

    private var countDownUtil: CountDownUtil? = null

    val toastStr: MutableLiveData<String> = MutableLiveData()

    val timeout: MutableLiveData<Boolean> = MutableLiveData()

    fun loadConfig(context: Context) = viewModelScope.launch {
        val common = context.getSharedPreferences("player_timer", Context.MODE_PRIVATE)
        val defaultType = common.getInt("type", 0)
        val defaultValue = common.getInt("value", 0)
        val defaultDate = common.getString("datetime", "20220831000000")!!

        val default = CountDown(
            courseId = courseId,
            type = defaultType,
            value = defaultValue,
            datetime = defaultDate,
        )
        val special = repository.findHasCountDown(courseId).first()

        if (special == null) {
            countDown = default
        } else {
            countDown = special
        }

        setCountDown(countDown)
    }

    /* 保存对该门课程单独设置的定时 */
    private fun saveCountDown(countDown: CountDown) = viewModelScope.launch(Dispatchers.IO) {
        repository.deleteCountDown(courseId)
        repository.insertCountDown(countDown)
    }

    /* 开启定时器 */
    private fun setCountDown(countDown: CountDown) {
        countDownUtil?.cancel()
        countDownUtil = null

        checkTimeout(countDown)
    }

    fun resumeTimer() {
        countDownUtil?.startOrResume()
    }

    fun pauseTimer() {
        countDownUtil?.pause()
    }

    /* 保存一条观看时间段的记录 */
    fun saveHistory(start: Long, end: Long, sectionId: String) {
        val history = History(
            start = start,
            end = end,
            courseId = courseId,
            sectionId = sectionId,
            date = SimpleDateFormat("yyyy-MM-dd").format(Date()),
            countDown = countDown.datetime,
        )

        viewModelScope.launch(Dispatchers.IO) {
            repository.insertHistory(history)
        }
    }

    suspend fun checkCanPlay(whenCan: () -> Unit) = viewModelScope.launch(Dispatchers.IO) {
        checkTimeout(countDown)

        withContext(Dispatchers.Main) {
            whenCan.invoke()
        }
    }

    override fun onListen(type: Int, value: Int) {
        Log.d("PVM", "onListen: $type , $value")

        countDown = CountDown(
            courseId = courseId,
            type = type,
            value = value,
            datetime = SimpleDateFormat("yyyyMMddhhmmss").format(Date())
        )

        countDown.let {
            setCountDown(it)
            saveCountDown(it)
        }
    }

    override fun onCleared() {
        super.onCleared()
        PlayerTimerUtil.removeListener(this)
    }

    override fun onTimeout(util: CountDownUtil?) {
        util?.cancel()
        sendTimeout(true)
    }

    private fun checkTimeout(countDown: CountDown) {
        countDown.also {
            viewModelScope.launch {
                if (it.type == CountDown.TYPE_DURATION) {
                    checkDurationTimeout()
                } else if (it.type == CountDown.TYPE_EPISODE) {
                    checkEpisodeTimeout()
                } else {
                    sendTimeout(false)
                }
            }
        }
    }

    private suspend fun checkEpisodeTimeout() = viewModelScope.launch(Dispatchers.IO) {
        val history = repository.queryTodayEpisodeHistory(courseId, countDown.datetime!!)

        if (history >= countDown.value) {
            sendTimeout(true)
        } else {
            sendTimeout(false)
        }
    }

    private suspend fun checkDurationTimeout() = viewModelScope.launch(Dispatchers.IO) {
        val history = repository.queryTodayDurationHistory(courseId, countDown.datetime!!)

        val result = history.fold(0L) { r, h ->
            r + (h.end - h.start)
        }

        if (result >= countDown.value) {
            sendTimeout(true)
        } else {
            sendTimeout(false)
            setupTimer(countDown.value - result)
        }
    }

    private fun setupTimer(count: Long) {
        if (countDown.type == CountDown.TYPE_DURATION) {
            countDownUtil = CountDownUtil.build(1000, count, this)
        }
    }

    private fun sendTimeout(state: Boolean) {
        viewModelScope.launch(Dispatchers.Main) {
            timeout.value = state
        }
    }

}

class PlayerViewModelFactory(
    private val courseId: String,
    private val repository: PlayerRepository,
) : ViewModelProvider.Factory {
    override fun <T : ViewModel> create(modelClass: Class<T>): T {
        if (modelClass.isAssignableFrom(PlayerViewModel::class.java)) {
            @Suppress("UNCHECKED_CAST")
            return PlayerViewModel(courseId, repository) as T
        }
        throw IllegalArgumentException("Unknown ViewModel class")
    }
}